<?php
/**
 * Jet_Search_Ajax_Handlers class
 *
 * @package   jet-search
 * @author    Zemez
 * @license   GPL-2.0+
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! class_exists( 'Jet_Search_Ajax_Handlers' ) ) {

	/**
	 * Define Jet_Search_Ajax_Handlers class
	 */
	class Jet_Search_Ajax_Handlers {

		/**
		 * A reference to an instance of this class.
		 *
		 * @since 1.0.0
		 * @var   Jet_Search_Ajax_Handlers
		 */
		private static $instance = null;

		/**
		 * Ajax action.
		 *
		 * @var string
		 */
		private $action = 'jet_ajax_search';

		/**
		 * Has navigation.
		 *
		 * @var bool
		 */
		public $has_navigation = false;

		/**
		 * Search query.
		 *
		 * @var array
		 */
		public $search_query = array();

		/**
		 * Constructor for the class
		 */
		public function init() {
			add_action( 'pre_get_posts', array( $this, 'set_search_query' ) );

			add_action( "wp_ajax_{$this->action}", array( $this, 'get_search_results' ) );
			add_action( "wp_ajax_nopriv_{$this->action}", array( $this, 'get_search_results' ) );

			jet_search_assets()->localize_data['ajaxurl'] = esc_url( admin_url( 'admin-ajax.php' ) );
			jet_search_assets()->localize_data['action']  = $this->action;
			jet_search_assets()->localize_data['nonce']   = wp_create_nonce( $this->action );
		}

		/**
		 * Set search query settings on the search result page.
		 *
		 * @param object $query
		 */
		public function set_search_query( $query ) {
			if ( ! is_admin() && $query->is_search && ! empty( $_GET['jet_ajax_search_settings'] ) ) {
				$form_settings = stripcslashes( $_GET['jet_ajax_search_settings'] );
				$form_settings = json_decode( $form_settings );
				$form_settings = get_object_vars( $form_settings );

				$this->set_query_settings( $form_settings );

				$query->query_vars = array_merge( $query->query_vars, $this->search_query );
			}
		}

		/**
		 * Get search results.
		 */
		public function get_search_results() {
			if ( ! isset( $_GET['nonce'] ) || ! wp_verify_nonce( $_GET['nonce'], $this->action ) ) {
				exit();
			}

			$data = $this->get_search_data();

			if ( empty( $data ) ) {
				exit();
			}

			wp_send_json( array(
				'data' => $data,
			) );
		}

		/**
		 * Get search data.
		 *
		 * @return array|bool
		 */
		public function get_search_data() {
			if ( empty( $_GET['data'] ) ) {
				return false;
			}

			$data                                      = $_GET['data'];
			$this->search_query['s']                   = urldecode( $data['value'] );
			$this->search_query['nopaging']            = false;
			$this->search_query['ignore_sticky_posts'] = false;
			$this->search_query['posts_per_page']      = ( int ) $data['limit_query_in_result_area'];
			$this->search_query['post_status']         = 'publish';

			$this->set_query_settings( $data );

			$search   = new WP_Query( $this->search_query );
			$response = array(
				'error'      => false,
				'post_count' => 0,
				'message'    => '',
				'posts'      => null,
			);

			if ( is_wp_error( $search ) ) {
				$response['error']   = true;
				$response['message'] = esc_html( $data['server_error'] );

				return $response;
			}

			if ( empty( $search->post_count ) ) {
				$response['message'] = esc_html( $data['negative_search'] );

				return $response;
			}

			$data['post_count'] = $search->post_count;
			$data['columns']    = ceil ( $data['post_count'] / $data['limit_query'] );

			$response['posts']              = array();
			$response['columns']            = $data['columns'];
			$response['limit_query']        = $data['limit_query'];
			$response['post_count']         = $data['post_count'];
			$response['results_navigation'] = $this->get_results_navigation( $data );

			foreach ( $search->posts as $key => $value ) {

				$response['posts'][ $key ] = array(
					'title'     => $value->post_title,
					'content'   => $this->get_post_content( $data, $value ),
					'thumbnail' => $this->get_post_thumbnail( $data, $value ),
					'link'      => esc_url( get_post_permalink( $value->ID ) ),
				);

				if ( ! $this->has_navigation && $key === $data['limit_query'] - 1 ) {
					break;
				}
			}

			return $response;
		}

		/**
		 * Set search query settings.
		 *
		 * @param array $args
		 */
		protected function set_query_settings( $args = array() ) {
			if ( $args ) {
				$this->search_query['cache_results'] = true;
				$this->search_query['post_type']     = $args['search_source'];
				$this->search_query['order']         = $args['results_order'];
				$this->search_query['orderby']       = $args['results_order_by'];
				$this->search_query['tax_query']     = array( array( 'relation' => 'AND' ) );

				if ( ! empty( $args['category__in'] ) ) {
					$tax = ! empty( $args['search_taxonomy'] ) ? $args['search_taxonomy'] : 'category';

					array_push(
						$this->search_query['tax_query'][0],
						array(
							'taxonomy' => $tax,
							'field'    => 'id',
							'operator' => 'IN',
							'terms'    => $args['category__in'],
						)
					);
				}
			}
		}

		/**
		 * Return post thumbnail.
		 *
		 * @param array  $data
		 * @param object $value
		 *
		 * @return string
		 */
		private function get_post_thumbnail( $data, $value ) {
			$visible = filter_var( $data['thumbnail_visible'], FILTER_VALIDATE_BOOLEAN );

			if ( $visible ) {
				$thumbnail_format = apply_filters( 'jet-search/ajax-search/thumbnail-html', '<div class="jet-ajax-search__item-thumbnail">%s</div>' );
				$thumbnail_size   = ! empty( $data['thumbnail_size'] ) ? $data['thumbnail_size'] : 'thumbnail';
				$thumbnail_html   = get_the_post_thumbnail( $value->ID, $thumbnail_size, array( 'class' => 'jet-ajax-search__item-thumbnail-img' ) );

				if ( ! $thumbnail_html ) {
					$thumbnail_html = apply_filters(
						'jet-search/ajax-search/thumbnail-placeholder-html',
						'<div class="jet-ajax-search__item-thumbnail-placeholder"></div>'
					);
				}

				$output_html = sprintf( $thumbnail_format, $thumbnail_html );

				return $output_html;
			}

			return '';
		}

		/**
		 * Return post content.
		 *
		 * @param array  $data
		 * @param object $value
		 *
		 * @return string
		 */
		private function get_post_content( $data, $value ) {
			$after  = '&hellip;';
			$length = ( int ) $data['post_content_length'];

			if ( 0 !== $length ) {
				$content = strip_shortcodes( $value->post_content );

				return wp_trim_words( $content, $length, $after );
			}

			return '';
		}

		/**
		 * Return result area navigation.
		 *
		 * @param array $settings
		 *
		 * @return array
		 */
		public function get_results_navigation( $settings = array() ) {
			$navigation_container_html = apply_filters( 'jet-search/ajax-search/navigation-container-html', '<div class="jet-ajax-search__navigation-container">%s</div>' );
			$navigation_types = apply_filters( 'jet-search/ajax-search/navigation-types', array( 'bullet_pagination', 'number_pagination', 'navigation_arrows' ) );
			$header_navigation = '';
			$footer_navigation = '';

			if ( $settings['limit_query'] < $settings['post_count'] ) {

				foreach ( $navigation_types as $type ) {
					if ( ! isset( $settings[ $type ] ) ) {
						continue;
					}

					if ( ! $settings[ $type ] ) {
						continue;
					}

					$buttons = $this->get_navigation_buttons_html( $settings, $type );

					if ( empty( $buttons ) ) {
						continue;
					}

					$this->has_navigation = true;

					switch ( $settings[ $type ] ) {
						case 'in_header':
							$header_navigation .= sprintf( $navigation_container_html, $buttons );
							break;

						case 'in_footer':
							$footer_navigation .= sprintf( $navigation_container_html, $buttons );
							break;

						case 'both':
							$header_navigation .= sprintf( $navigation_container_html, $buttons );
							$footer_navigation .= sprintf( $navigation_container_html, $buttons );
							break;
					}
				}
			}

			return array(
				'in_header' => $header_navigation,
				'in_footer' => $footer_navigation,
			);
		}

		/**
		 * Get results navigation buttons html.
		 *
		 * @param array  $settings
		 * @param string $type
		 *
		 * @return string
		 */
		public function get_navigation_buttons_html( $settings = array(), $type = 'bullet_pagination' ) {
			$output_html = '';
			$bullet_html = apply_filters( 'jet-search/ajax-search/navigate-button-html', '<button class="jet-ajax-search__navigate-button %1$s" data-number="%2$s"></button>' );

			switch ( $type ) {
				case 'bullet_pagination':
					$button_class = 'jet-ajax-search__bullet-button';

				case 'number_pagination':
					$button_class = isset( $button_class ) ? $button_class : 'jet-ajax-search__number-button';

					for ( $i = 0; $i < $settings['columns']; $i++ ) {
						$active_button_class = ( $i === 0 ) ? ' jet-ajax-search__active-button' : '' ;
						$output_html .= sprintf( $bullet_html, $button_class . $active_button_class, $i + 1 );
					}
					break;

				case 'navigation_arrows':
					$prev_button = apply_filters( 'jet-search/ajax-search/prev-button-html', '<button class="jet-ajax-search__prev-button jet-ajax-search__arrow-button jet-ajax-search__navigate-button jet-ajax-search__navigate-button-disable %s" data-direction="-1"></button>' );
					$next_button = apply_filters( 'jet-search/ajax-search/next-button-html', '<button class="jet-ajax-search__next-button jet-ajax-search__arrow-button jet-ajax-search__navigate-button %s" data-direction="1"></button>' );
					$output_html = sprintf( $prev_button . $next_button, esc_attr( $settings['navigation_arrows_type'] ), esc_attr( $settings['navigation_arrows_type'] ) );
					break;
			}

			return $output_html;
		}

		/**
		 * Returns the instance.
		 *
		 * @since  1.0.0
		 * @return Jet_Search_Ajax_Handlers
		 */
		public static function get_instance() {

			// If the single instance hasn't been set, set it now.
			if ( null == self::$instance ) {
				self::$instance = new self;
			}
			return self::$instance;
		}
	}
}

/**
 * Returns instance of Jet_Search_Ajax_Handlers
 *
 * @return Jet_Search_Ajax_Handlers
 */
function jet_search_ajax_handlers() {
	return Jet_Search_Ajax_Handlers::get_instance();
}
